/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.datamodel.*;

import jalview.jbgui.*;

import java.awt.event.*;

import java.util.*;

import javax.swing.event.*;
import jalview.util.Comparison;
import javax.swing.*;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision$
 */
public class RedundancyPanel extends GSliderPanel implements Runnable
{
    AlignFrame af;
    AlignmentPanel ap;
    Stack historyList = new Stack(); // simpler than synching with alignFrame.
    float [] redundancy;
    Vector originalSequences;
    Hashtable originalColours;
    JInternalFrame frame;

    /**
     * Creates a new RedundancyPanel object.
     *
     * @param ap DOCUMENT ME!
     * @param af DOCUMENT ME!
     */
    public RedundancyPanel(AlignmentPanel ap, AlignFrame af)
    {
        this.ap = ap;
        this.af = af;

        slider.addChangeListener(new ChangeListener()
            {
                public void stateChanged(ChangeEvent evt)
                {
                    valueField.setText(slider.getValue() + "");
                    sliderValueChanged();
                }
            });

        applyButton.setText("Remove");
        allGroupsCheck.setVisible(false);
        slider.setMinimum(60);
        slider.setMaximum(100);
        slider.setValue(100);

        Thread worker = new Thread(this);
        worker.start();

        frame = new JInternalFrame();
        frame.setContentPane(this);
        Desktop.addInternalFrame(frame, "Redundancy threshold selection", 400,
                         100, false);
        frame.addInternalFrameListener(new InternalFrameAdapter()
            {
              public void internalFrameClosing(InternalFrameEvent evt)
              {
                resetColours();
              }
            }
            );

    }


    /**
     * This is a copy of remove redundancy in jalivew.datamodel.Alignment
     * except we dont want to remove redundancy, just calculate once
     * so we can use the slider to dynamically hide redundant sequences
     *
     * @param threshold DOCUMENT ME!
     * @param sel DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public void run()
    {
        JProgressBar progress = new JProgressBar();
        progress.setIndeterminate(true);
        southPanel.add(progress, java.awt.BorderLayout.SOUTH);

        label.setText("Calculating....");

        slider.setVisible(false);
        applyButton.setEnabled(false);
        valueField.setVisible(false);

        validate();

        Vector sel = new Vector();
        SequenceGroup sg = ap.av.getSelectionGroup();
        int height;
        originalSequences = new Vector();
        originalColours = new Hashtable();

        if ( (sg != null) && (sg.getSize() >= 1))
        {
           height = sg.getSize();
          for (int i = 0; i < sg.getSize(); i++)
          {
            sel.addElement(sg.getSequenceAt(i));
          }
        }
        else
        {
          height = ap.av.alignment.getHeight();
          for (int i = 0; i < ap.av.alignment.getHeight(); i++)
          {
            sel.addElement(ap.av.alignment.getSequenceAt(i));
          }
        }

        redundancy = new float[height];
        for (int i = 0; i < height; i++)
        {
          redundancy[i] = 0f;
        }


       // long start = System.currentTimeMillis();

        float pid;
        SequenceI seqi, seqj;
        for (int i = 0; i < sel.size(); i++)
        {
            originalSequences.addElement(sel.elementAt(i));
            originalColours.put(sel.elementAt(i),
                               ((SequenceI) sel.elementAt(i)).getColor());

            for (int j = 0; j < i; j++)
            {
              if(i==j)
                continue;

              seqi = (SequenceI) sel.elementAt(i);
              seqj = (SequenceI) sel.elementAt(j);

              if (sg != null)
                pid = Comparison.PID( seqi,
                                     seqj,
                                     sg.getStartRes(), sg.getEndRes());
              else
                pid = Comparison.PID( seqi,  seqj );


              if(seqj.getLength() < seqi.getLength())
                redundancy[j] = Math.max(pid, redundancy[j]);
              else
                redundancy[i] = Math.max(pid, redundancy[i]);

            }
        }


        progress.setIndeterminate(false);
        progress.setVisible(false);
        progress = null;

        label.setText("Enter the redundancy threshold");
        slider.setVisible(true);
        applyButton.setEnabled(true);
        valueField.setVisible(true);

        validate();
       // System.out.println("blob done "+ (System.currentTimeMillis()-start));
    }

    void sliderValueChanged()
    {
      if(redundancy==null)
        return;

      float value = slider.getValue();

      for(int i=0; i<redundancy.length; i++)
      {
        if (value > redundancy[i])
           ((SequenceI)originalSequences.elementAt(i)).setColor(java.awt.Color.white);
        else
           ((SequenceI)originalSequences.elementAt(i)).setColor(java.awt.Color.red);
      }

      PaintRefresher.Refresh(null,ap.av.alignment);

    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void applyButton_actionPerformed(ActionEvent e)
    {
        historyList.push(new HistoryItem("Remove redundancy",
                    ap.av.alignment, HistoryItem.HIDE));

        if ((historyList.size() == 1) ||
                !af.historyList.contains(historyList.firstElement()))
        {
            af.addHistoryItem((HistoryItem) historyList.firstElement());
            af.updateEditMenuBar();
        }

        Vector del = new Vector();

        undoButton.setEnabled(true);

        float value = slider.getValue();
        SequenceGroup sg = ap.av.getSelectionGroup();

        for (int i = 0; i < redundancy.length; i++)
        {
          if (value <= redundancy[i])
          {
            SequenceI seq = (SequenceI) originalSequences.elementAt(i);
            ap.av.alignment.deleteSequence(seq);
            del.add(seq);
            if (sg != null)
            {
              sg.deleteSequence(seq, false);
            }
          }
        }


        // This has to be done before the restoreHistoryItem method of alignFrame will
        // actually restore these sequences.
        if (del.size() > 0)
        {
            for (int i = 0, j = del.size(); i < j; i++)
            {
                SequenceI sq = (SequenceI) del.elementAt(i);
                sq.deleteChars(0, sq.getLength());
            }
        }

        ap.av.firePropertyChange("alignment", null, ap.av.getAlignment().getSequences());
        af.updateEditMenuBar();
    }

    void resetColours()
    {
      for(int i=0; i<originalSequences.size(); i++)
      {
        SequenceI seq = (SequenceI)originalSequences.elementAt(i);
        seq.setColor( (java.awt.Color)originalColours.get(seq));
      }

      PaintRefresher.Refresh(ap.av.alignment);
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void undoButton_actionPerformed(ActionEvent e)
    {
      HistoryItem hi = (HistoryItem) historyList.pop();
      af.restoreHistoryItem(hi);

      if (historyList.size() == 0)
      {
        undoButton.setEnabled(false);

        if (af.historyList.contains(hi))
        {
          af.historyList.remove(hi);
          af.updateEditMenuBar();
        }
      }
    }

    /**
     * DOCUMENT ME!
     *
     * @param e DOCUMENT ME!
     */
    public void valueField_actionPerformed(ActionEvent e)
    {
        try
        {
            int i = Integer.parseInt(valueField.getText());
            slider.setValue(i);
        }
        catch (Exception ex)
        {
            valueField.setText(slider.getValue() + "");
        }
    }


}
